/* libc/sys/linux/include/unistd.h - Various standard functions */

/* Written 2000 by Werner Almesberger */


#ifndef _NEWLIB_UNISTD_H
#define _NEWLIB_UNISTD_H

#include <sys/types.h>
#include <sys/unistd.h>

/* Declare some missing goodies */

extern char *optarg;
extern int optind, opterr, optopt;


#endif
